/*
 parseTree.c è un modulo che fornisce l'implementazione di 3 nuovi tipi di dato.
 Il tipo parseTree con le seguenti operazioni (di interfaccia):
 - makeEmpty
 - makeLeaf
 - makeRooted
 - isEmpty
 - isLeaf
 - isRooted
 - getLabel
 - getSons
 - printTree
 Il tipo treeList con le seguenti operazioni (di interfaccia)
 - nil
 - cons
 - split
 Il tipo bool senza operazioni con due valori literal
 + false
 + true
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "parseTree.h"

//Le operazioni: Costruttori di treeList
treeList nil(){
    return NULL;
}
treeList cons(parseTree val, treeList list){
    treeList r;
    r = (treeList) malloc(sizeof(treeListElem));
    r->currentSon = val;
    r->next = list;
    return r;
}
//Le operazioni: Ispettori e Selettori di treeList
bool isNull(treeList list){
    return (list==NULL);
}
bool hd(treeList list, parseTree * val){
    // Trattamento eccezione lista vuota: False se lista vuota
    if (!isNull(list)) {
        * val = list->currentSon;
        return (true);
    }
    return(false);
}
bool tail(treeList list, treeList * val){
    // Trattamento eccezione lista vuota: False se lista vuota
    if (!isNull(list)) {
        * val = list->next;
        return (true);
    }
    return(false);
}
bool split(treeList list, parseTree * treeVal, treeList * listVal){
    // Trattamento eccezione lista vuota: False se lista vuota
    if (!isNull(list)) {
        *treeVal = list->currentSon;
        *listVal = list->next;
        return (true);
    }
    return(false);
}
//Le operazioni: Costruttori di parseTree
parseTree makeEmpty(){
    return NULL;
}
parseTree makeLeaf(int labelKind, char * etichetta){
    parseTree r;
    r = (parseTree) malloc(sizeof(parseTreeElem));
    r->treeKind = 0;
    r->etichetta = etichetta;
    r->labelKind = labelKind;
    r->sons = NULL;
    return r;
}
parseTree makeRooted(char * etichetta, treeList sons){
    parseTree r;
    r = (parseTree) malloc(sizeof(parseTreeElem));
    r->treeKind = 1;
    r->etichetta = etichetta;
    r->labelKind = 1;
    r->sons = sons;
    return r;
}
//Le operazioni: Ispettori e Selettori di parseTree
bool isEmpty(parseTree t){
    return (t==NULL);
}
bool isLeaf(parseTree t){
    return (!isEmpty(t))&&(t->treeKind == 0);
}
bool isRooted(parseTree t){
    return (!isEmpty(t))&&(t->treeKind == 1);
}
char * getLabelOld(parseTree t){//solo per leaf e rooted parseTree
    if (isEmpty(t)) return NULL;
    return (t->etichetta);
}
bool getLabel(parseTree t, char** r){//usabile sempre
        //eccezione segnalata dal boolean restituito
        //valore calcolato restituito con parametro aggiuntivo r.
    if (isEmpty(t)) return false;
    *r = t->etichetta;
    return true;
}
treeList getSons(parseTree t){//solo per rooted parseTree
    if (isEmpty(t)||isLeaf(t)) return NULL;
    return (t->sons);
}
//-- operazioni per la stampa --
void printTree(parseTree t);
void printTreeSons(treeList sons){
    parseTree leftmost;
    treeList others;
    if (split(sons,&leftmost,&others)){
        printTree(leftmost);
        printTreeSons(others);
        return;
    }
    return;
}
void printlnTree(parseTree t){
    printTree(t); printf("\n");
}
void printTree(parseTree t){
    if (isEmpty(t)) {
        printf("[]"); return;
    }
    if (isLeaf(t)) {
        printf("[%s]",t->etichetta); return;
    }
    printf("[%s-(",t->etichetta);
    printTreeSons(t->sons);
    printf(")]");
    return;
}
